@@ -27,6 +27,7 @@
 #include "volumefadereffect.h"
 #include <gst/interfaces/propertyprobe.h>
 #include <phonon/pulsesupport.h>
+#include <phonon/GlobalDescriptionContainer>
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QSet>
@@ -67,7 +68,7 @@ Backend::Backend(QObject *parent, const QVariantList &)
 
     QByteArray appFilePath = qApp->applicationFilePath().toUtf8();
     QByteArray gstDebugLevel("--gst-debug-level=");
-    gstDebugLevel.append(qgetenv("PHONON_GST_GST_DEBUG"));
+    gstDebugLevel.append(qgetenv("PHONON_SUBSYSTEM_DEBUG"));
 
     const char *args[] = {
         appFilePath.constData(),
@@ -92,7 +93,7 @@ Backend::Backend(QObject *parent, const QVariantList &)
 #endif //QT_NO_PROPERTIES
 
     //check if we should enable debug output
-    QString debugLevelString = qgetenv("PHONON_GST_DEBUG");
+    QString debugLevelString = qgetenv("PHONON_BACKEND_DEBUG");
     int debugLevel = debugLevelString.toInt();
     if (debugLevel > 3) //3 is maximum
         debugLevel = 3;
@@ -113,9 +114,12 @@ Backend::Backend(QObject *parent, const QVariantList &)
 
 Backend::~Backend()
 {
+    if (GlobalSubtitles::self)
+        delete GlobalSubtitles::self;
     delete m_effectManager;
     delete m_deviceManager;
     PulseSupport::shutdown();
+    gst_deinit();
 }
 
 /***
@@ -173,7 +177,7 @@ bool Backend::supportsVideo() const
     return isValid();
 }
 
-bool Backend::checkDependencies() const
+bool Backend::checkDependencies(bool retry) const
 {
     bool success = false;
     // Verify that gst-plugins-base is installed
@@ -186,11 +190,19 @@ bool Backend::checkDependencies() const
         if (csFactory) {
             gst_object_unref(csFactory);
         } else {
+            if (!retry) {
+                gst_update_registry();
+                checkDependencies(true);
+            }
             QString message = tr("Warning: You do not seem to have the package gstreamer0.10-plugins-good installed.\n"
                                  "          Some video features have been disabled.");
             qDebug() << message;
         }
     } else {
+        if (!retry) {
+            gst_update_registry();
+            checkDependencies(true);
+        }
         qWarning() << tr("Warning: You do not seem to have the base GStreamer plugins installed.\n"
                          "          All audio and video support has been disabled");
     }
@@ -213,6 +225,7 @@ QStringList Backend::availableMimeTypes() const
         (mpegFactory = gst_element_factory_find ("mad")) ||
         (mpegFactory = gst_element_factory_find ("flump3dec"))) {
         availableMimeTypes << QLatin1String("audio/x-mp3");
+        availableMimeTypes << QLatin1String("audio/x-ape");// ape is available from ffmpeg
         gst_object_unref(GST_OBJECT(mpegFactory));
     }
 
@@ -301,6 +314,10 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
             break;
         }
         break;
+    case Phonon::SubtitleType: {
+            list << GlobalSubtitles::instance()->globalIndexes();
+        }
+        break;
     default:
         break;
     }
@@ -352,6 +369,16 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
             } else
                 Q_ASSERT(1); // Since we use list position as ID, this should not happen
         }
+        break;
+
+    case Phonon::SubtitleType: {
+            const SubtitleDescription description = GlobalSubtitles::instance()->fromIndex(index);
+            ret.insert("name", description.name());
+            ret.insert("description", description.description());
+            ret.insert("type", description.property("type"));
+        }
+        break;
+
     default:
         break;
     }
@@ -384,7 +411,6 @@ bool Backend::connectNodes(QObject *source, QObject *sink)
         MediaNode *sinkNode = qobject_cast<MediaNode *>(sink);
         if (sourceNode && sinkNode) {
             if (sourceNode->connectNode(sink)) {
-                sourceNode->root()->invalidateGraph();
                 logMessage(QString("Backend connected %0 to %1").arg(source->metaObject()->className()).arg(sink->metaObject()->className()));
                 return true;
             }
@@ -436,7 +462,7 @@ EffectManager* Backend::effectManager() const
 
 /**
  * Returns a debuglevel that is determined by the
- * PHONON_GST_DEBUG environment variable.
+ * PHONON_BACKEND_DEBUG environment variable.
  *
  *  Warning - important warnings
  *  Info    - general info
@@ -455,6 +481,10 @@ Backend::DebugLevel Backend::debugLevel() const
  */
 void Backend::logMessage(const QString &message, int priority, QObject *obj) const
 {
+    // Backend is a singleton, so this is just fine.
+    static QString lastLogMessage = QString();
+    static int logMessageSkipCount = 0;
+    
     if (debugLevel() > 0) {
         QString output;
         if (obj) {
@@ -469,9 +499,14 @@ void Backend::logMessage(const QString &message, int priority, QObject *obj) con
         else {
             output = message;
         }
-        if (priority <= (int)debugLevel()) {
+        if (priority <= (int)debugLevel() && lastLogMessage != output) {
+            if (logMessageSkipCount != 0) 
+                qDebug() << "  PGST: Last message repeated" << logMessageSkipCount << "time(s)";
             qDebug() << QString("PGST(%1): %2").arg(priority).arg(output);
-        }
+            lastLogMessage = output;
+            logMessageSkipCount = 0;
+        } else
+            ++logMessageSkipCount;
     }
 }
 
